/**
 * <p>File: TestSystemSolver.java</p>
 * <p>Title: Numerical methods</p>
 * <p>Description: Course on numerical analysis</p>
 * <p>Date: June 2009</p>
 * @author Faton M. Berisha
 * @version 4.01
 */
import numeric.*;

public class TestSystemSolver {
	private static class TestVectorFunction implements VectorFunction {
		public double value(int i, double[] x) {
			double answer = 0;
			switch (i) {
			case 0: {
				answer = Math.cos(x[1] * x[2]) / 3.0 + 1.0 / 6.0;
				break;
			}
			case 1: {
				answer = Math.sqrt(x[0] * x[0] + Math.sin(x[2]) + 1.06) / 9.0 - 0.1;
				break;
			}
			case 2: {
				answer = -Math.exp(-x[0] * x[1]) / 20.0 - (10 * Math.PI - 3)
						/ 60.0;
				break;
			}
			}
			return answer;
		}
	}

	private static class TestPartiallyDifferentiable implements PartiallyDifferentiable {
		public double value(int i, double[] x) {
			double answer = 0;
			switch (i) {
			case 0:
				answer = 3 * x[0] - Math.cos(x[1] * x[2]) - 1.0 / 2.0;
				break;
			case 1:
				answer = x[0] * x[0] - 81 * (x[1] + 0.1) * (x[1] + 0.1)
						+ Math.sin(x[2]) + 1.06;
				break;
			case 2:
				answer = Math.exp(-x[0] * x[1]) + 20 * x[2]
						+ (10 * Math.PI - 3) / 3;
				break;
			}
			return answer;
		}
		
		public double partialDerivative(int i, int j, double[] x) {
			double answer = 0;
			switch (i) {
			case 0:
				switch(j) {
				case 0:
					answer = 3;
					break;
				case 1:
					answer = x[2] * Math.sin(x[1] * x[2]);
					break;
				case 2:
					answer = x[1] * Math.sin(x[1] * x[2]);
					break;
				}
				break;
			case 1:
				switch(j) {
				case 0:
					answer = 2 * x[0];
					break;
				case 1:
					answer = -162 * (x[1] + 0.1);
					break;
				case 2:
					answer = Math.cos(x[2]);
					break;
				}
				break;
			case 2:
				switch(j) {
				case 0:
					answer = -x[1] * Math.exp(-x[0] * x[1]);
					break;
				case 1:
					answer = -x[0] * Math.exp(-x[0] * x[1]);
					break;
				case 2:
					answer = 20;
					break;
				}
				break;
			}
			return answer;
		}
	}

	public static void main(String[] args) {
		double[] initial = { 0.1, 0.1, -0.1 };
		SystemSolver solver = new SystemSolver(initial.length);

		// Example 10.1.2
		solver.setTolerance(1E-8);
		solver.setNumIterations(20);
		SystemDemoView.setNumSignificantDigits(8);
		VectorFunction g = new TestVectorFunction();
		new SystemDemoView("Metoda e piks fikse", solver);
		try {
			solver.fixedPoint(g, initial);
		} catch (ArithmeticException e) {
		}

		// Example 10.1.3
		PartiallyDifferentiable f = new TestPartiallyDifferentiable();
		new SystemDemoView("Metoda e Newton-it", solver);
		try {
			solver.newton(f, initial);
		} catch (ArithmeticException e) {
			System.err.println(e.getMessage());
		}

	}
}
